unsigned char A, X, Y, SP, Op;
unsigned int  PC, PCd, PCe;

struct pf
{
    unsigned int C : 1;     /* Carry             */
    unsigned int Z : 1;     /* Zero              */
    unsigned int I : 1;     /* Interrupt disable */
    unsigned int D : 1;     /* Decimal mode      */
    unsigned int B : 1;     /* Break             */
    unsigned int X : 1;     /* reserved          */
    unsigned int V : 1;     /* oVerflow          */
    unsigned int S : 1;     /* Sign or negative  */
    unsigned int Q : 8;
};

union
{
    struct pf F;
    unsigned int P;
} P;


struct Instructions
{
    unsigned char object;
    char          length;
    void (*Process)( void );
    char          flag;
    char          *asmx;
} Code[] =
{
    0x00, 0, BRK, 0, "BRK",
    0x01, 1, ORA, 0, "ORA (#%02X,X)",
    0x02, 0, XXX, 0, "???",
    0x03, 0, XXX, 0, "???",
    0x04, 0, XXX, 0, "???",
    0x05, 1, ORA, 0, "ORA #%02X",
    0x06, 1, ASL, 0, "ASL #%02X",
    0x07, 0, XXX, 0, "???",
    0x08, 0, PHP, 0, "PHP",
    0x09, 1, ORA, 0, "ORA @#%02X",
    0x0A, 0, ASL, 0, "ASL A",
    0x0B, 0, XXX, 0, "???",
    0x0C, 0, XXX, 0, "???",
    0x0D, 2, ORA, 0, "ORA #%04X",
    0x0E, 2, ASL, 0, "ASL #%04X",
    0x0F, 0, XXX, 0, "???",
    0x10, 1, Bfl, 1, "BPL #%02X",
    0x11, 1, ORA, 0, "ORA (#%02X),Y",
    0x12, 0, XXX, 0, "???",
    0x13, 0, XXX, 0, "???",
    0x14, 0, XXX, 0, "???",
    0x15, 1, ORA, 0, "ORA #%02X,X",
    0x16, 1, ASL, 0, "ASL #%02X,X",
    0x17, 0, XXX, 0, "???",
    0x18, 0, SCf, 0, "CLC",
    0x19, 2, ORA, 0, "ORA #%04X,Y",
    0x1A, 0, XXX, 0, "???",
    0x1B, 0, XXX, 0, "???",
    0x1C, 0, XXX, 0, "???",
    0x1D, 2, ORA, 0, "ORA #%04X,X",
    0x1E, 2, ASL, 0, "ASL #%04X,X",
//    0x1F, 2, ORA, 0, "ORA #%04X,Y",
    0x1F, 0, XXX, 0, "???",
    0x20, 2, JSR, 0, "JSR #%04X",
    0x21, 1, AND, 0, "AND (#%02X,X)",
    0x22, 0, XXX, 0, "???",
    0x23, 0, XXX, 0, "???",
    0x24, 1, BIT, 0, "BIT #%02X",
    0x25, 1, AND, 0, "AND #%02X",
    0x26, 1, ROL, 0, "ROL #%02X",
    0x27, 0, XXX, 0, "???",
    0x28, 0, PLP, 0, "PLP",
    0x29, 1, AND, 0, "AND @#%02X",
    0x2A, 0, ROL, 0, "ROL A",
    0x2B, 0, XXX, 0, "???",
    0x2C, 2, BIT, 0, "BIT #%04X",
    0x2D, 2, AND, 0, "AND #%04X",
    0x2E, 2, ROL, 0, "ROL #%04X",
    0x2F, 0, XXX, 0, "???",
    0x30, 1, Bfl, 1, "BMI #%02X",
    0x31, 1, AND, 0, "AND (#%02X),Y",
    0x32, 0, XXX, 0, "???",
    0x33, 0, XXX, 0, "???",
    0x34, 0, XXX, 0, "???",
    0x35, 1, AND, 0, "AND #%02X,X",
    0x36, 1, ROL, 0, "ROL #%02X,X",
    0x37, 0, XXX, 0, "???",
    0x38, 0, SCf, 0, "SEC",
    0x39, 2, AND, 0, "AND #%04X,Y",
    0x3A, 0, XXX, 0, "???",
    0x3B, 0, XXX, 0, "???",
    0x3C, 0, XXX, 0, "???",
    0x3D, 2, AND, 0, "AND #%04X,X",
    0x3E, 2, ROL, 0, "ROL #%04X,X",
    0x3F, 0, XXX, 0, "???",
    0x40, 0, RTI, 0, "RTI",
    0x41, 1, EOR, 0, "EOR (#%02X,X)",
    0x42, 0, XXX, 0, "???",
    0x43, 0, XXX, 0, "???",
    0x44, 0, XXX, 0, "???",
    0x45, 1, EOR, 0, "EOR #%02X",
    0x46, 1, LSR, 0, "LSR #%02X",
    0x47, 0, XXX, 0, "???",
    0x48, 0, PHA, 0, "PHA",
    0x49, 1, EOR, 0, "EOR @#%02X",
    0x4A, 0, LSR, 0, "LSR A",
    0x4B, 0, XXX, 0, "???",
    0x4C, 2, JMP, 0, "JMP #%04X",
    0x4D, 2, EOR, 0, "EOR #%04X",
    0x4E, 2, LSR, 0, "LSR #%04X",
    0x4F, 0, XXX, 0, "???",
    0x50, 1, Bfl, 1, "BVC #%02X",
    0x51, 1, EOR, 0, "EOR (#%02X),Y",
    0x52, 0, XXX, 0, "???",
    0x53, 0, XXX, 0, "???",
    0x54, 0, XXX, 0, "???",
    0x55, 1, EOR, 0, "EOR #%02X,X",
    0x56, 1, LSR, 0, "LSR #%02X,X",
    0x57, 0, XXX, 0, "???",
    0x58, 0, SCf, 0, "CLI",
    0x59, 2, EOR, 0, "EOR #%04X,Y",
    0x5A, 0, XXX, 0, "???",
    0x5B, 0, XXX, 0, "???",
    0x5C, 0, XXX, 0, "???",
    0x5D, 2, EOR, 0, "EOR #%04X,X",
    0x5E, 2, LSR, 0, "LSR #%04X,X",
    0x5F, 0, XXX, 0, "???",
    0x60, 0, RTS, 0, "RTS",
    0x61, 1, ADC, 0, "ADC (#%02X,X)",
    0x62, 0, XXX, 0, "???",
    0x63, 0, XXX, 0, "???",
    0x64, 0, XXX, 0, "???",
    0x65, 1, ADC, 0, "ADC #%02X",
    0x66, 1, ROR, 0, "ROR #%02X",
    0x67, 0, XXX, 0, "???",
    0x68, 0, PLA, 0, "PLA",
    0x69, 1, ADC, 0, "ADC @#%02X",
    0x6A, 0, ROR, 0, "ROR A",
    0x6B, 0, XXX, 0, "???",
    0x6C, 2, JMP, 0, "JMP (#%04X)",
    0x6D, 2, ADC, 0, "ADC #%04X",
    0x6E, 2, ROR, 0, "ROR #%04X",
    0x6F, 0, XXX, 0, "???",
    0x70, 1, Bfl, 1, "BVS #%02X",
    0x71, 1, ADC, 0, "ADC (#%02X),Y",
    0x72, 0, XXX, 0, "???",
    0x73, 0, XXX, 0, "???",
    0x74, 0, XXX, 0, "???",
    0x75, 1, ADC, 0, "ADC #%02X,X",
    0x76, 1, ROR, 0, "ROR #%02X,X",
    0x77, 0, XXX, 0, "???",
    0x78, 0, SCf, 0, "SEI",
    0x79, 2, ADC, 0, "ADC #%04X,Y",
    0x7A, 0, XXX, 0, "???",
    0x7B, 0, XXX, 0, "???",
    0x7C, 0, XXX, 0, "???",
    0x7D, 2, ADC, 0, "ADC #%04X,X",
    0x7E, 2, ROR, 0, "ROR #%04X,X",
    0x7F, 0, XXX, 0, "???",
    0x80, 0, XXX, 0, "???",
    0x81, 1, STA, 0, "STA (#%02X,X)",
    0x82, 0, XXX, 0, "???",
    0x83, 0, XXX, 0, "???",
    0x84, 1, STY, 0, "STY #%02X",
    0x85, 1, STA, 0, "STA #%02X",
    0x86, 1, STX, 0, "STX #%02X",
    0x87, 0, XXX, 0, "???",
    0x88, 0, DEY, 0, "DEY",
    0x89, 0, XXX, 0, "???",
    0x8A, 0, TXA, 0, "TXA",
    0x8B, 0, XXX, 0, "???",
    0x8C, 2, STY, 0, "STY #%04X",
    0x8D, 2, STA, 0, "STA #%04X",
    0x8E, 2, STX, 0, "STX #%04X",
    0x8F, 0, XXX, 0, "???",
    0x90, 1, Bfl, 1, "BCC #%02X",
    0x91, 1, STA, 0, "STA (#%02X),Y",
    0x92, 0, XXX, 0, "???",
    0x93, 0, XXX, 0, "???",
    0x94, 1, STY, 0, "STY #%02X,X",
    0x95, 1, STA, 0, "STA #%02X,X",
    0x96, 1, STX, 0, "STX #%02X,Y",
    0x97, 0, XXX, 0, "???",
    0x98, 0, TYA, 0, "TYA",
    0x99, 2, STA, 0, "STA #%04X,Y",
    0x9A, 0, TXS, 0, "TXS",
    0x9B, 0, XXX, 0, "???",
    0x9C, 0, XXX, 0, "???",
    0x9D, 2, STA, 0, "STA #%04X,X",
    0x9E, 0, XXX, 0, "???",
    0x9F, 0, XXX, 0, "???",
    0xA0, 1, LDY, 0, "LDY @#%02X",
    0xA1, 1, LDA, 0, "LDA (#%02X,X)",
    0xA2, 1, LDX, 0, "LDX @#%02X",
    0xA3, 0, XXX, 0, "???",
    0xA4, 1, LDY, 0, "LDY #%02X",
    0xA5, 1, LDA, 0, "LDA #%02X",
    0xA6, 1, LDX, 0, "LDX #%02X",
    0xA7, 0, XXX, 0, "???",
    0xA8, 0, TAY, 0, "TAY",
    0xA9, 1, LDA, 0, "LDA @#%02X",
    0xAA, 0, TAX, 0, "TAX",
    0xAB, 0, XXX, 0, "???",
    0xAC, 2, LDY, 0, "LDY #%04X",
    0xAD, 2, LDA, 0, "LDA #%04X",
    0xAE, 2, LDX, 0, "LDX #%04X",
    0xAF, 0, XXX, 0, "???",
    0xB0, 1, Bfl, 1, "BCS #%02X",
    0xB1, 1, LDA, 0, "LDA (#%02X),Y",
    0xB2, 0, XXX, 0, "???",
    0xB3, 0, XXX, 0, "???",
    0xB4, 1, LDY, 0, "LDY #%02X,X",
    0xB5, 1, LDA, 0, "LDA #%02X,X",
    0xB6, 1, LDX, 0, "LDX #%02X,Y",
    0xB7, 0, XXX, 0, "???",
    0xB8, 0, SCf, 0, "CLV",
    0xB9, 2, LDA, 0, "LDA #%04X,Y",
    0xBA, 0, TSX, 0, "TSX",
    0xBB, 0, XXX, 0, "???",
    0xBC, 2, LDY, 0, "LDY #%04X,X",
    0xBD, 2, LDA, 0, "LDA #%04X,X",
    0xBE, 2, LDX, 0, "LDX #%04X,Y",
    0xBF, 0, XXX, 0, "???",
    0xC0, 1, CPr, 0, "CPY @#%02X",
    0xC1, 1, CMP, 0, "CMP (#%02X,X)",
    0xC2, 0, XXX, 0, "???",
    0xC3, 0, XXX, 0, "???",
    0xC4, 1, CPr, 0, "CPY #%02X",
    0xC5, 1, CMP, 0, "CMP #%02X",
    0xC6, 1, DEC, 0, "DEC #%02X",
    0xC7, 0, XXX, 0, "???",
    0xC8, 0, INY, 0, "INY",
    0xC9, 1, CMP, 0, "CMP @#%02X",
    0xCA, 0, DEX, 0, "DEX",
    0xCB, 0, XXX, 0, "???",
    0xCC, 2, CPr, 0, "CPY #%04X",
    0xCD, 2, CMP, 0, "CMP #%04X",
    0xCE, 2, DEC, 0, "DEC #%04X",
    0xCF, 0, XXX, 0, "???",
    0xD0, 1, Bfl, 1, "BNE #%02X",
    0xD1, 1, CMP, 0, "CMP (#%02X),Y",
    0xD2, 0, XXX, 0, "???",
    0xD3, 0, XXX, 0, "???",
    0xD4, 0, XXX, 0, "???",
    0xD5, 1, CMP, 0, "CMP #%02X,X",
    0xD6, 1, DEC, 0, "DEC #%02X,X",
    0xD7, 0, XXX, 0, "???",
    0xD8, 0, SCf, 0, "CLD",
    0xD9, 2, CMP, 0, "CMP #%04X,Y",
    0xDA, 0, XXX, 0, "???",
    0xDB, 0, XXX, 0, "???",
    0xDC, 0, XXX, 0, "???",
    0xDD, 2, CMP, 0, "CMP #%04X,X",
    0xDE, 2, DEC, 0, "DEC #%04X,X",
    0xDF, 0, XXX, 0, "???",
    0xE0, 1, CPr, 0, "CPX @#%02X",
    0xE1, 1, SBC, 0, "SBC (#%02X,X)",
    0xE2, 0, XXX, 0, "???",
    0xE3, 0, XXX, 0, "???",
    0xE4, 1, CPr, 0, "CPX #%02X",
    0xE5, 1, SBC, 0, "SBC #%02X",
    0xE6, 1, INC, 0, "INC #%02X",
    0xE7, 0, XXX, 0, "???",
    0xE8, 0, INX, 0, "INX",
    0xE9, 1, SBC, 0, "SBC @#%02X",
    0xEA, 0, NOP, 0, "NOP",
    0xEB, 0, XXX, 0, "???",
    0xEC, 2, CPr, 0, "CPX #%04X",
    0xED, 2, SBC, 0, "SBC #%04X",
    0xEE, 2, INC, 0, "INC #%04X",
    0xEF, 0, XXX, 0, "???",
    0xF0, 1, Bfl, 1, "BEQ #%02X",
    0xF1, 1, SBC, 0, "SBC (#%02X),Y",
    0xF2, 0, XXX, 0, "???",
    0xF3, 0, XXX, 0, "???",
    0xF4, 0, XXX, 0, "???",
    0xF5, 1, SBC, 0, "SBC #%02X,X",
    0xF6, 1, INC, 0, "INC #%02X,X",
    0xF7, 0, XXX, 0, "???",
    0xF8, 0, SCf, 0, "SED",
    0xF9, 2, SBC, 0, "SBC #%04X,Y",
    0xFA, 0, XXX, 0, "???",
    0xFB, 0, XXX, 0, "???",
    0xFC, 0, XXX, 0, "???",
    0xFD, 2, SBC, 0, "SBC #%04X,X",
    0xFE, 2, INC, 0, "INC #%04X,X",
    0xFF, 0, XXX, 0, "???"
};
