unsigned char A, X, Y, SP, Op;
unsigned int  PC, PCd, PCe;

struct pf
{
    unsigned int C : 1;     /* Carry             */
    unsigned int Z : 1;     /* Zero              */
    unsigned int I : 1;     /* Interrupt disable */
    unsigned int D : 1;     /* Decimal mode      */
    unsigned int B : 1;     /* Break             */
    unsigned int X : 1;     /* reserved          */
    unsigned int V : 1;     /* oVerflow          */
    unsigned int S : 1;     /* Sign or negative  */
    unsigned int Q : 8;
};

union
{
    struct pf F;
    unsigned int P;
} P;


struct Instructions
{
    unsigned char object;
    char          length;
    void (*Process)( void );
    char          flag;
    char          *asmx;
} Code[] =
{
    0x00, 0, BRK , 0, "BRK",
    0x01, 1, ORA0, 0, "ORA (#%02X,X)",
    0x02, 0, XXX , 0, "???",
    0x03, 0, XXX , 0, "???",
    0x04, 0, XXX , 0, "???",
    0x05, 1, ORA1, 0, "ORA #%02X",
    0x06, 1, ASL1, 0, "ASL #%02X",
    0x07, 0, XXX , 0, "???",
    0x08, 0, PHP , 0, "PHP",
    0x09, 1, ORA2, 0, "ORA @#%02X",
    0x0A, 0, ASLA, 0, "ASL A",
    0x0B, 0, XXX , 0, "???",
    0x0C, 0, XXX , 0, "???",
    0x0D, 2, ORA3, 0, "ORA #%04X",
    0x0E, 2, ASL3, 0, "ASL #%04X",
    0x0F, 0, XXX , 0, "???",
    0x10, 1, BPL , 1, "BPL #%02X",
    0x11, 1, ORA4, 0, "ORA (#%02X),Y",
    0x12, 0, XXX , 0, "???",
    0x13, 0, XXX , 0, "???",
    0x14, 0, XXX , 0, "???",
    0x15, 1, ORA5, 0, "ORA #%02X,X",
    0x16, 1, ASL5, 0, "ASL #%02X,X",
    0x17, 0, XXX , 0, "???",
    0x18, 0, CLC , 0, "CLC",
    0x19, 2, ORA6, 0, "ORA #%04X,Y",
    0x1A, 0, XXX , 0, "???",
    0x1B, 0, XXX , 0, "???",
    0x1C, 0, XXX , 0, "???",
    0x1D, 2, ORA7, 0, "ORA #%04X,X",
    0x1E, 2, ASL7, 0, "ASL #%04X,X",
    0x1F, 0, XXX , 0, "???",
    0x20, 2, JSR , 0, "JSR #%04X",
    0x21, 1, AND0, 0, "AND (#%02X,X)",
    0x22, 0, XXX , 0, "???",
    0x23, 0, XXX , 0, "???",
    0x24, 1, BIT0, 0, "BIT #%02X",
    0x25, 1, AND1, 0, "AND #%02X",
    0x26, 1, ROL1, 0, "ROL #%02X",
    0x27, 0, XXX , 0, "???",
    0x28, 0, PLP , 0, "PLP",
    0x29, 1, AND2, 0, "AND @#%02X",
    0x2A, 0, ROLA, 0, "ROL A",
    0x2B, 0, XXX , 0, "???",
    0x2C, 2, BIT1, 0, "BIT #%04X",
    0x2D, 2, AND3, 0, "AND #%04X",
    0x2E, 2, ROL3, 0, "ROL #%04X",
    0x2F, 0, XXX , 0, "???",
    0x30, 1, BMI , 1, "BMI #%02X",
    0x31, 1, AND4, 0, "AND (#%02X),Y",
    0x32, 0, XXX , 0, "???",
    0x33, 0, XXX , 0, "???",
    0x34, 0, XXX , 0, "???",
    0x35, 1, AND5, 0, "AND #%02X,X",
    0x36, 1, ROL5, 0, "ROL #%02X,X",
    0x37, 0, XXX , 0, "???",
    0x38, 0, SEC , 0, "SEC",
    0x39, 2, AND6, 0, "AND #%04X,Y",
    0x3A, 0, XXX , 0, "???",
    0x3B, 0, XXX , 0, "???",
    0x3C, 0, XXX , 0, "???",
    0x3D, 2, AND7, 0, "AND #%04X,X",
    0x3E, 2, ROL7, 0, "ROL #%04X,X",
    0x3F, 0, XXX , 0, "???",
    0x40, 0, RTI , 0, "RTI",
    0x41, 1, EOR0, 0, "EOR (#%02X,X)",
    0x42, 0, XXX , 0, "???",
    0x43, 0, XXX , 0, "???",
    0x44, 0, XXX , 0, "???",
    0x45, 1, EOR1, 0, "EOR #%02X",
    0x46, 1, LSR1, 0, "LSR #%02X",
    0x47, 0, XXX , 0, "???",
    0x48, 0, PHA , 0, "PHA",
    0x49, 1, EOR2, 0, "EOR @#%02X",
    0x4A, 0, LSRA, 0, "LSR A",
    0x4B, 0, XXX , 0, "???",
    0x4C, 2, JMP0, 0, "JMP #%04X",
    0x4D, 2, EOR3, 0, "EOR #%04X",
    0x4E, 2, LSR3, 0, "LSR #%04X",
    0x4F, 0, XXX , 0, "???",
    0x50, 1, BVC , 1, "BVC #%02X",
    0x51, 1, EOR4, 0, "EOR (#%02X),Y",
    0x52, 0, XXX , 0, "???",
    0x53, 0, XXX , 0, "???",
    0x54, 0, XXX , 0, "???",
    0x55, 1, EOR5, 0, "EOR #%02X,X",
    0x56, 1, LSR5, 0, "LSR #%02X,X",
    0x57, 0, XXX , 0, "???",
    0x58, 0, CLI , 0, "CLI",
    0x59, 2, EOR6, 0, "EOR #%04X,Y",
    0x5A, 0, XXX , 0, "???",
    0x5B, 0, XXX , 0, "???",
    0x5C, 0, XXX , 0, "???",
    0x5D, 2, EOR7, 0, "EOR #%04X,X",
    0x5E, 2, LSR7, 0, "LSR #%04X,X",
    0x5F, 0, XXX , 0, "???",
    0x60, 0, RTS , 0, "RTS",
    0x61, 1, ADC0, 0, "ADC (#%02X,X)",
    0x62, 0, XXX , 0, "???",
    0x63, 0, XXX , 0, "???",
    0x64, 0, XXX , 0, "???",
    0x65, 1, ADC1, 0, "ADC #%02X",
    0x66, 1, ROR1, 0, "ROR #%02X",
    0x67, 0, XXX , 0, "???",
    0x68, 0, PLA , 0, "PLA",
    0x69, 1, ADC2, 0, "ADC @#%02X",
    0x6A, 0, RORA, 0, "ROR A",
    0x6B, 0, XXX , 0, "???",
    0x6C, 2, JMP1, 0, "JMP (#%04X)",
    0x6D, 2, ADC3, 0, "ADC #%04X",
    0x6E, 2, ROR3, 0, "ROR #%04X",
    0x6F, 0, XXX , 0, "???",
    0x70, 1, BVS , 1, "BVS #%02X",
    0x71, 1, ADC4, 0, "ADC (#%02X),Y",
    0x72, 0, XXX , 0, "???",
    0x73, 0, XXX , 0, "???",
    0x74, 0, XXX , 0, "???",
    0x75, 1, ADC5, 0, "ADC #%02X,X",
    0x76, 1, ROR5, 0, "ROR #%02X,X",
    0x77, 0, XXX , 0, "???",
    0x78, 0, SEI , 0, "SEI",
    0x79, 2, ADC6, 0, "ADC #%04X,Y",
    0x7A, 0, XXX , 0, "???",
    0x7B, 0, XXX , 0, "???",
    0x7C, 0, XXX , 0, "???",
    0x7D, 2, ADC7, 0, "ADC #%04X,X",
    0x7E, 2, ROR7, 0, "ROR #%04X,X",
    0x7F, 0, XXX , 0, "???",
    0x80, 0, XXX , 0, "???",
    0x81, 1, STA0, 0, "STA (#%02X,X)",
    0x82, 0, XXX , 0, "???",
    0x83, 0, XXX , 0, "???",
    0x84, 1, STY1, 0, "STY #%02X",
    0x85, 1, STA1, 0, "STA #%02X",
    0x86, 1, STX1, 0, "STX #%02X",
    0x87, 0, XXX , 0, "???",
    0x88, 0, DEY , 0, "DEY",
    0x89, 0, XXX , 0, "???",
    0x8A, 0, TXA , 0, "TXA",
    0x8B, 0, XXX , 0, "???",
    0x8C, 2, STY3, 0, "STY #%04X",
    0x8D, 2, STA3, 0, "STA #%04X",
    0x8E, 2, STX3, 0, "STX #%04X",
    0x8F, 0, XXX , 0, "???",
    0x90, 1, BCC , 1, "BCC #%02X",
    0x91, 1, STA4, 0, "STA (#%02X),Y",
    0x92, 0, XXX , 0, "???",
    0x93, 0, XXX , 0, "???",
    0x94, 1, STY5, 0, "STY #%02X,X",
    0x95, 1, STA5, 0, "STA #%02X,X",
    0x96, 1, STX5, 0, "STX #%02X,Y",
    0x97, 0, XXX , 0, "???",
    0x98, 0, TYA , 0, "TYA",
    0x99, 2, STA6, 0, "STA #%04X,Y",
    0x9A, 0, TXS , 0, "TXS",
    0x9B, 0, XXX , 0, "???",
    0x9C, 0, XXX , 0, "???",
    0x9D, 2, STA7, 0, "STA #%04X,X",
    0x9E, 0, XXX , 0, "???",
    0x9F, 0, XXX , 0, "???",
    0xA0, 1, LDY0, 0, "LDY @#%02X",
    0xA1, 1, LDA0, 0, "LDA (#%02X,X)",
    0xA2, 1, LDX0, 0, "LDX @#%02X",
    0xA3, 0, XXX , 0, "???",
    0xA4, 1, LDY1, 0, "LDY #%02X",
    0xA5, 1, LDA1, 0, "LDA #%02X",
    0xA6, 1, LDX1, 0, "LDX #%02X",
    0xA7, 0, XXX , 0, "???",
    0xA8, 0, TAY , 0, "TAY",
    0xA9, 1, LDA2, 0, "LDA @#%02X",
    0xAA, 0, TAX , 0, "TAX",
    0xAB, 0, XXX , 0, "???",
    0xAC, 2, LDY3, 0, "LDY #%04X",
    0xAD, 2, LDA3, 0, "LDA #%04X",
    0xAE, 2, LDX3, 0, "LDX #%04X",
    0xAF, 0, XXX , 0, "???",
    0xB0, 1, BCS , 1, "BCS #%02X",
    0xB1, 1, LDA4, 0, "LDA (#%02X),Y",
    0xB2, 0, XXX , 0, "???",
    0xB3, 0, XXX , 0, "???",
    0xB4, 1, LDY5, 0, "LDY #%02X,X",
    0xB5, 1, LDA5, 0, "LDA #%02X,X",
    0xB6, 1, LDX5, 0, "LDX #%02X,Y",
    0xB7, 0, XXX , 0, "???",
    0xB8, 0, CLV , 0, "CLV",
    0xB9, 2, LDA6, 0, "LDA #%04X,Y",
    0xBA, 0, TSX , 0, "TSX",
    0xBB, 0, XXX , 0, "???",
    0xBC, 2, LDY7, 0, "LDY #%04X,X",
    0xBD, 2, LDA7, 0, "LDA #%04X,X",
    0xBE, 2, LDX7, 0, "LDX #%04X,Y",
    0xBF, 0, XXX , 0, "???",
    0xC0, 1, CPY0, 0, "CPY @#%02X",
    0xC1, 1, CMP0, 0, "CMP (#%02X,X)",
    0xC2, 0, XXX , 0, "???",
    0xC3, 0, XXX , 0, "???",
    0xC4, 1, CPY1, 0, "CPY #%02X",
    0xC5, 1, CMP1, 0, "CMP #%02X",
    0xC6, 1, DEC1, 0, "DEC #%02X",
    0xC7, 0, XXX , 0, "???",
    0xC8, 0, INY , 0, "INY",
    0xC9, 1, CMP2, 0, "CMP @#%02X",
    0xCA, 0, DEX , 0, "DEX",
    0xCB, 0, XXX , 0, "???",
    0xCC, 2, CPY3, 0, "CPY #%04X",
    0xCD, 2, CMP3, 0, "CMP #%04X",
    0xCE, 2, DEC3, 0, "DEC #%04X",
    0xCF, 0, XXX , 0, "???",
    0xD0, 1, BNE , 1, "BNE #%02X",
    0xD1, 1, CMP4, 0, "CMP (#%02X),Y",
    0xD2, 0, XXX , 0, "???",
    0xD3, 0, XXX , 0, "???",
    0xD4, 0, XXX , 0, "???",
    0xD5, 1, CMP5, 0, "CMP #%02X,X",
    0xD6, 1, DEC5, 0, "DEC #%02X,X",
    0xD7, 0, XXX , 0, "???",
    0xD8, 0, CLD , 0, "CLD",
    0xD9, 2, CMP6, 0, "CMP #%04X,Y",
    0xDA, 0, XXX , 0, "???",
    0xDB, 0, XXX , 0, "???",
    0xDC, 0, XXX , 0, "???",
    0xDD, 2, CMP7, 0, "CMP #%04X,X",
    0xDE, 2, DEC7, 0, "DEC #%04X,X",
    0xDF, 0, XXX , 0, "???",
    0xE0, 1, CPX0, 0, "CPX @#%02X",
    0xE1, 1, SBC0, 0, "SBC (#%02X,X)",
    0xE2, 0, XXX , 0, "???",
    0xE3, 0, XXX , 0, "???",
    0xE4, 1, CPX1, 0, "CPX #%02X",
    0xE5, 1, SBC1, 0, "SBC #%02X",
    0xE6, 1, INC1, 0, "INC #%02X",
    0xE7, 0, XXX , 0, "???",
    0xE8, 0, INX , 0, "INX",
    0xE9, 1, SBC2, 0, "SBC @#%02X",
    0xEA, 0, NOP , 0, "NOP",
    0xEB, 0, XXX , 0, "???",
    0xEC, 2, CPX3, 0, "CPX #%04X",
    0xED, 2, SBC3, 0, "SBC #%04X",
    0xEE, 2, INC3, 0, "INC #%04X",
    0xEF, 0, XXX , 0, "???",
    0xF0, 1, BEQ , 1, "BEQ #%02X",
    0xF1, 1, SBC4, 0, "SBC (#%02X),Y",
    0xF2, 0, XXX , 0, "???",
    0xF3, 0, XXX , 0, "???",
    0xF4, 0, XXX , 0, "???",
    0xF5, 1, SBC5, 0, "SBC #%02X,X",
    0xF6, 1, INC5, 0, "INC #%02X,X",
    0xF7, 0, XXX , 0, "???",
    0xF8, 0, SED , 0, "SED",
    0xF9, 2, SBC6, 0, "SBC #%04X,Y",
    0xFA, 0, XXX , 0, "???",
    0xFB, 0, XXX , 0, "???",
    0xFC, 0, XXX , 0, "???",
    0xFD, 2, SBC7, 0, "SBC #%04X,X",
    0xFE, 2, INC7, 0, "INC #%04X,X",
    0xFF, 0, XXX , 0, "???"
};
