#include <stdio.h>
#include <io.h>
#include <ctype.h>
#include <string.h>
#include <conio.h>
#include <bios.h>
#include <dos.h>
#include <alloc.h>
#include <stdlib.h>

#include "proto9.h"
#include "6502_9.h"
#include "grafix2.h"

enum
{
    READ = 0,
    WRITE,
    FALSE = 0,
    TRUE,
    IRQ = 0,
    NMI
};

extern int FKey, cb, CrtOutput, nmi_set, Stop, Continue, BreakDetect;
extern void interrupt (*org)();

void ProcessOpcode( void )
{
    unsigned register addr, tmp, w1, w2;

    while (!Stop)
    {
        if (FKey == TRUE)               /* Do only if Function Key pressed */
        {
            if (cb == 0x43)
            {
                while(cb == 0x43)           /* F9  Atom Break */
                    ;
                Init_6502();
            }
            if (cb == 0x3D)                 /* F3  Switch crt 32 - 80 */
            {
                while(cb == 0x3D)
                    ;
                if ( CrtOutput == FALSE )
                    SwitchCrt( TRUE );
                else
                    SwitchCrt( FALSE );
            }
            if (cb == 0x3C)                 /* F2  MENU */
            {
                if (!nmi_set)
                {
                    while(cb == 0x3C)
                        ;
                    setvect(0x09, org);
                    FileMenu();
                    setvect(0x09, kbint);
                    cb = 0x00;
                }
            }
            if (cb == 0x3B)                 /* F1  Help menu */
            {
                if (!nmi_set)
                {
                    while(cb == 0x3C)
                        ;
                    setvect(0x09, org);
                    HelpMenu();
                    setvect(0x09, kbint);
                    cb = 0x00;
                }
            }
        }
        if (nmi_set)                                        /* NMI Occured */
            if (!P.F.I)
                INT(NMI);

/***********************  Process 6502 Instruction ******************/

        Op = MRead( PC++ );

        switch(Op)
        {
            case 0x00 :                                     /* BREAK */
                P.F.B = 1;
                PC++;
                if (BreakDetect && !Continue)
                {
                    Stop = 1;
                }
                else
                {
                    Continue = 0;
                    MWrite( 0x100 + SP--, PC >> 8 );
                    MWrite( 0x100 + SP--, PC & 0xFF );
                    MWrite( 0x100 + SP--, P.P  & 0xFF );
                    PC = MRead( 0xFFFE ) + ((MRead( 0xFFFF )) << 8);
                }
                P.F.I = 1;
                break;
            case 0x01 :                                     /* ORA 0 */
                addr = MRead( PC++ ) + X;
                A |= MRead( MRead( addr ) + (MRead( addr + 1 ) << 8) );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x02 :
                break;
            case 0x03 :
                break;
            case 0x04 :
                break;
            case 0x05 :                                     /* ORA 1 */
                A |= MRead( MRead( PC++ ) );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x06 :                                     /* ASL 1 */
                addr = MRead( PC++ );
                addr = MRead( PC++ );
                tmp = (MRead( addr )) << 1;
                MWrite( addr, tmp & 0xFF);
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.C = (tmp & 0x100) ? 1 : 0;
                break;
            case 0x07 :
                break;
            case 0x08 :                                     /* PHP */
                MWrite( 0x100 + SP--, P.P);
                break;
            case 0x09 :                                     /* ORA 2 */
                A |= MRead( PC++ );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x0A :                                     /* ASL A */
                tmp = A << 1;
                A = tmp & 0xFF;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.C = (tmp & 0x100) ? 1 : 0;
                break;
            case 0x0B :
                break;
            case 0x0C :
                break;
            case 0x0D :                                     /* ORA 3 */
                A |= MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x0E :                                     /* ASL 3 */
                addr = MRead( PC++ ) + ((MRead( PC++ )) << 8);
                tmp = (MRead( addr )) << 1;
                MWrite( addr, tmp & 0xFF);
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.C = (tmp & 0x100) ? 1 : 0;
                break;
            case 0x0F :
                break;
            case 0x10 :                                     /* BPL */
                if (!P.F.S)
                    PC += MRead( PC );
                PC++;
                break;
            case 0x11 :                                     /* ORA 4 */
                addr = MRead( PC++ );
                A |= MRead( MRead( addr ) + (MRead( addr + 1 ) << 8) + Y );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x12 :
                break;
            case 0x13 :
                break;
            case 0x14 :
                break;
            case 0x15 :                                     /* ORA 5 */
                A |= MRead( MRead( PC++ ) + X );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x16 :                                     /* ASL 5 */
                addr = MRead( PC++ ) + ((MRead( PC++ )) << 8) + X;
                tmp = (MRead( addr )) << 1;
                MWrite( addr, tmp & 0xFF);
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.C = (tmp & 0x100) ? 1 : 0;
                break;
            case 0x17 :
                break;
            case 0x18 :                                     /* CLC */
                P.F.C = 0;
                break;
            case 0x19 :                                     /* ORA 6 */
                A |= MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) + Y );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x1A :
                break;
            case 0x1B :
                break;
            case 0x1C :
                break;
            case 0x1D :                                     /* ORA 7 */
                A |= MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) + X );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x1E :                                     /* ASL 7 */
                addr = MRead( PC++ ) + ((MRead( PC++ )) << 8) + X;
                tmp = (MRead( addr )) << 1;
                MWrite( addr, tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.C = (tmp & 0x100) ? 1 : 0;
                break;
            case 0x1F :
                break;
            case 0x20 :                                     /* JMP */
                PC = MRead( PC++ ) + (MRead( PC ) << 8);
                break;
            case 0x21 :                                     /* AND 0 */
                addr = MRead( PC++ ) + X;
                A &= MRead( MRead( addr ) + (MRead( addr + 1 ) << 8) );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x22 :
                break;
            case 0x23 :
                break;
            case 0x24 :                                     /* BIT 0 */
                tmp = MRead( MRead( PC++ ) );
                P.F.Z = !(A & tmp);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.V = (tmp & 0x40) ? 1 : 0;
                break;
            case 0x25 :                                     /* AND 1 */
                A &= MRead( MRead( PC++ ) );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x26 :                                     /* ROL 1 */
                addr = MRead( PC++ );
                tmp = (MRead( addr )) << 1;
                tmp |= P.F.C;
                MWrite( addr, tmp & 0xFF);
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.C = (tmp & 0x100) ? 1 : 0;
                break;
            case 0x27 :
                break;
            case 0x28 :                                     /* PLP */
                P.P = MRead( 0x100 + ++SP );
                break;
            case 0x29 :                                     /* AND 2 */
                A &= MRead( PC++ );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x2A :                                     /* ROL A */
                tmp = A << 1;
                tmp |= P.F.C;
                A = tmp & 0xFF;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.C = (tmp & 0x100) ? 1 : 0;
                break;
            case 0x2B :
                break;
            case 0x2C :                                     /* BIT 1 */
                tmp = MRead( MRead( PC++ ) + ((MRead( PC++ )) << 8) );
                P.F.Z = !(A & tmp);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.V = (tmp & 0x40) ? 1 : 0;
                break;
            case 0x2D :                                     /* AND 3 */
                A &= MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) );
                A &= MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x2E :                                     /* ROL 3 */
                addr = MRead( PC++ ) + ((MRead( PC++ )) << 8);
                tmp = (MRead( addr )) << 1;
                tmp |= P.F.C;
                MWrite( addr, tmp & 0xFF);
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.C = (tmp & 0x100) ? 1 : 0;
                break;
            case 0x2F :
                break;
            case 0x30 :                                     /* BMI */
                if (P.F.S)
                    PC += MRead( PC );
                PC++;
                break;
            case 0x31 :                                     /* AND 4 */
                addr = MRead( PC++ );
                A &= MRead( MRead( addr ) + (MRead( addr + 1 ) << 8) + Y );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x32 :
                break;
            case 0x33 :
                break;
            case 0x34 :
                break;
            case 0x35 :                                     /* AND 5 */
                A &= MRead( MRead( PC++ ) + X );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x36 :                                     /* ROL 5 */
                addr = MRead( PC++ ) + X;
                tmp = (MRead( addr )) << 1;
                tmp |= P.F.C;
                MWrite( addr, tmp & 0xFF);
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.C = (tmp & 0x100) ? 1 : 0;
                break;
            case 0x37 :
                break;
            case 0x38 :                                     /* SEC */
                P.F.C = 1;
                break;
            case 0x39 :                                     /* AND 6 */
                A &= MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) + Y );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x3A :
                break;
            case 0x3B :
                break;
            case 0x3C :
                break;
            case 0x3D :                                     /* AND 7 */
                A &= MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) + X );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x3E :                                     /* ROL 7 */
                addr = MRead( PC++ ) + ((MRead( PC++ )) << 8) + X;
                tmp = (MRead( addr )) << 1;
                tmp |= P.F.C;
                MWrite( addr, tmp & 0xFF);
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.C = (tmp & 0x100) ? 1 : 0;
                break;
            case 0x3F :
                break;
            case 0x40 :                                     /* RTI */
                P.P = MRead( 0x100 + ++SP );
                PC = MRead( 0x100 + ++SP ) + (MRead( 0x100 + ++SP ) << 8);
                break;
            case 0x41 :                                     /* EOR 0 */
                addr = MRead( PC++ ) + X;
                A ^= MRead( MRead( addr ) + (MRead( addr + 1 ) << 8) );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x42 :
                break;
            case 0x43 :
                break;
            case 0x44 :
                break;
            case 0x45 :                                     /* EOR 1 */
                A ^= MRead( MRead( PC++ ) );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x46 :                                     /* LSR 1 */
                addr = MRead( PC++ );
                tmp = MRead( addr );
                P.F.C = tmp & 0x01;
                tmp >>= 1;
                MWrite( addr, tmp & 0xFF);
                P.F.Z = !(tmp & 0xFF);
                P.F.S = 0;
                break;
            case 0x47 :
                break;
            case 0x48 :                                     /* PHA */
                MWrite( 0x100 + SP--, A);                       /* PUSH */
                break;
            case 0x49 :                                     /* EOR 2 */
                A ^= MRead( PC++ );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x4A :                                     /* LSR A */
                P.F.C = A & 0x01;
                A >>= 1;
                P.F.Z = !A;
                P.F.S = 0;
                break;
            case 0x4B :
                break;
            case 0x4C :                                     /* JMP 0 */
                PC = MRead( PC++ ) + (MRead( PC ) << 8);
                break;
            case 0x4D :                                     /* EOR 3 */
                A ^= MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x4E :                                     /* LSR 3 */
                addr = MRead( PC++ ) + ((MRead( PC++ )) << 8);
                tmp = MRead( addr );
                P.F.C = tmp & 0x01;
                tmp >>= 1;
                MWrite( addr, tmp & 0xFF);
                P.F.Z = !(tmp & 0xFF);
                P.F.S = 0;
                break;
            case 0x4F :
                break;
            case 0x50 :                                     /* BVC */
                if (!P.F.V)
                    PC += MRead( PC );
                PC++;
                break;
            case 0x51 :                                     /* EOR 4 */
                addr = MRead( PC++ );
                A ^= MRead( MRead( addr ) + (MRead( addr + 1 ) << 8) + Y );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x52 :
                break;
            case 0x53 :
                break;
            case 0x54 :
                break;
            case 0x55 :                                     /* EOR 5 */
                A ^= MRead( MRead( PC++ ) + X );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x56 :                                     /* LSR 5 */
                addr = MRead( PC++ ) + X;
                tmp = MRead( addr );
                P.F.C = tmp & 0x01;
                tmp >>= 1;
                MWrite( addr, tmp & 0xFF);
                P.F.Z = !(tmp & 0xFF);
                P.F.S = 0;
                break;
            case 0x57 :
                break;
            case 0x58 :                                     /* CLI */
                P.F.I = 0;
                break;
            case 0x59 :                                     /* EOR 6 */
                A ^= MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) + Y );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x5A :
                break;
            case 0x5B :
                break;
            case 0x5C :
                break;
            case 0x5D :                                     /* EOR 7 */
                A ^= MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) + X );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x5E :                                     /* LSR 7 */
                addr = MRead( PC++ ) + ((MRead( PC++ )) << 8) + X;
                tmp = MRead( addr );
                P.F.C = tmp & 0x01;
                tmp >>= 1;
                MWrite( addr, tmp & 0xFF);
                P.F.Z = !(tmp & 0xFF);
                P.F.S = 0;
                break;
            case 0x5F :
                break;
            case 0x60 :                                     /* RTS */
                PC = MRead( 0x100 + ++SP ) + (MRead( 0x100 + ++SP ) << 8);
                PC++;
                break;
            case 0x61 :                                     /* ADC 0 */
                addr = MRead( PC++ ) + X;
                tmp = A + MRead( MRead( addr ) + (MRead( addr + 1 ) << 8) ) + P.F.C;
                if (P.F.D)                /* Decimal Mode */
                {
                    if ((tmp & 0x0F) > 0x09)
                        tmp += 0x06;
                    if ((tmp & 0xF0) > 0x90)
                        tmp += 0x60;
                }
                P.F.V = ((A & 0xC0) == 0x40) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 1 : 0;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = P.F.C ^ P.F.V;
                break;
            case 0x62 :
                break;
            case 0x63 :
                break;
            case 0x64 :
                break;
            case 0x65 :                                     /* ADC 1 */
                tmp = A + MRead( MRead( PC++ ) ) + P.F.C;
                if (P.F.D)                /* Decimal Mode */
                {
                    if ((tmp & 0x0F) > 0x09)
                        tmp += 0x06;
                    if ((tmp & 0xF0) > 0x90)
                        tmp += 0x60;
                }
                P.F.V = ((A & 0xC0) == 0x40) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 1 : 0;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = P.F.C ^ P.F.V;
                break;
            case 0x66 :                                     /* ROR 1 */
                addr = MRead( PC++ );
                tmp = MRead( addr );
                P.F.X = tmp & 0x01;
                tmp >>= 1;
                tmp |= (P.F.C) ? 0x80 : 0x00;
                MWrite( addr, tmp & 0xFF);
                P.F.C = P.F.X;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                break;
            case 0x67 :
                break;
            case 0x68 :                                     /* PLA */
                A = MRead( 0x100 + ++SP );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0x69 :                                     /* ADC 2 */
                tmp = A + MRead( PC++ ) + P.F.C;
                if (P.F.D)                /* Decimal Mode */
                {
                    if ((tmp & 0x0F) > 0x09)
                        tmp += 0x06;
                    if ((tmp & 0xF0) > 0x90)
                        tmp += 0x60;
                }
                P.F.V = ((A & 0xC0) == 0x40) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 1 : 0;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = P.F.C ^ P.F.V;
                break;
            case 0x6A :                                     /* ROR A */
                tmp = A >> 1;
                P.F.X = A & 0x01;
                tmp |= (P.F.C) ? 0x80 : 0x00;
                A = tmp & 0xFF;
                P.F.C = P.F.X;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                break;
            case 0x6B :
                break;
            case 0x6C :                                     /* JMP 1 */
                addr = MRead( PC++ ) + (MRead( PC ) << 8);
                PC = MRead( addr++ ) + (MRead( addr ) << 8);
                break;
            case 0x6D :                                     /* ADC 3 */
                tmp = A + MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) ) + P.F.C;
                if (P.F.D)                /* Decimal Mode */
                {
                    if ((tmp & 0x0F) > 0x09)
                        tmp += 0x06;
                    if ((tmp & 0xF0) > 0x90)
                        tmp += 0x60;
                }
                P.F.V = ((A & 0xC0) == 0x40) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 1 : 0;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = P.F.C ^ P.F.V;
                break;
            case 0x6E :                                     /* ROR 3 */
                addr = MRead( PC++ ) + ((MRead( PC++ )) << 8);
                tmp = MRead( addr );
                P.F.X = tmp & 0x01;
                tmp >>= 1;
                tmp |= (P.F.C) ? 0x80 : 0x00;
                MWrite( addr, tmp & 0xFF);
                P.F.C = P.F.X;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                break;
            case 0x6F :
                break;
            case 0x70 :                                     /* BVS */
                if (P.F.V)
                    PC += MRead( PC );
                PC++;
                break;
            case 0x71 :                                     /* ADC 4 */
                addr = MRead( PC++ );
                tmp = A + MRead( MRead( addr ) + (MRead( addr + 1 ) << 8) + Y ) + P.F.C;
                if (P.F.D)                /* Decimal Mode */
                {
                    if ((tmp & 0x0F) > 0x09)
                        tmp += 0x06;
                    if ((tmp & 0xF0) > 0x90)
                        tmp += 0x60;
                }
                P.F.V = ((A & 0xC0) == 0x40) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 1 : 0;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = P.F.C ^ P.F.V;
                break;
            case 0x72 :
                break;
            case 0x73 :
                break;
            case 0x74 :
                break;
            case 0x75 :                                     /* ADC 5 */
                tmp = A + MRead( MRead( PC++ ) + X ) + P.F.C;
                if (P.F.D)                /* Decimal Mode */
                {
                    if ((tmp & 0x0F) > 0x09)
                        tmp += 0x06;
                    if ((tmp & 0xF0) > 0x90)
                        tmp += 0x60;
                }
                P.F.V = ((A & 0xC0) == 0x40) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 1 : 0;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = P.F.C ^ P.F.V;
                break;
            case 0x76 :                                     /* ROR 5 */
                addr = MRead( PC++ ) + X;
                tmp = MRead( addr );
                P.F.X = tmp & 0x01;
                tmp >>= 1;
                tmp |= (P.F.C) ? 0x80 : 0x00;
                MWrite( addr, tmp & 0xFF);
                P.F.C = P.F.X;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                break;
            case 0x77 :
                break;
            case 0x78 :                                     /* SEI */
                P.F.I = 1;
                break;
            case 0x79 :                                     /* ADC 6 */
                tmp = A + MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) + Y ) + P.F.C;
                if (P.F.D)                /* Decimal Mode */
                {
                    if ((tmp & 0x0F) > 0x09)
                        tmp += 0x06;
                    if ((tmp & 0xF0) > 0x90)
                        tmp += 0x60;
                }
                P.F.V = ((A & 0xC0) == 0x40) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 1 : 0;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = P.F.C ^ P.F.V;
                break;
            case 0x7A :
                break;
            case 0x7B :
                break;
            case 0x7C :
                break;
            case 0x7D :                                     /* ADC 7 */
                tmp = A + MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) + X ) + P.F.C;
                if (P.F.D)                /* Decimal Mode */
                {
                    if ((tmp & 0x0F) > 0x09)
                        tmp += 0x06;
                    if ((tmp & 0xF0) > 0x90)
                        tmp += 0x60;
                }
                P.F.V = ((A & 0xC0) == 0x40) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 1 : 0;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = P.F.C ^ P.F.V;
                break;
            case 0x7E :                                     /* ROR 7 */
                addr = MRead( PC++ ) + ((MRead( PC++ )) << 8) + X;
                tmp = MRead( addr );
                P.F.X = tmp & 0x01;
                tmp >>= 1;
                tmp |= (P.F.C) ? 0x80 : 0x00;
                MWrite( addr, tmp & 0xFF);
                P.F.C = P.F.X;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                break;
            case 0x7F :
                break;
            case 0x80 :
                break;
            case 0x81 :                                     /* STA 0 */
                addr = MRead( PC++ ) + X; /*?????????????????????????????*/
                addr = MRead( PC++ ) + X;
                MWrite( MRead( addr ) + (MRead( addr + 1 ) << 8), A );
                break;
            case 0x82 :
                break;
            case 0x83 :
                break;
            case 0x84 :                                     /* STY 1 */
                MWrite( MRead( PC++ ), Y );
                break;
            case 0x85 :                                     /* STA 1 */
                MWrite( MRead( PC++ ), A );
                break;
            case 0x86 :                                     /* STX 1 */
                MWrite( MRead( PC++ ), X );
                break;
            case 0x87 :
                break;
            case 0x88 :                                     /* DEY */
                Y--;
                P.F.S = (Y & 0x80) ? 1 : 0;
                P.F.Z = !Y;
                break;
            case 0x89 :
                break;
            case 0x8A :                                     /* TXA */
                A = X;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.Z = !A;
                break;
            case 0x8B :
                break;
            case 0x8C :                                     /* STY 3 */
                MWrite( MRead( PC++ ) + (MRead( PC++ ) << 8 ), Y );
                break;
            case 0x8D :                                     /* STA 3 */
                MWrite( MRead( PC++ ) + (MRead( PC++ ) << 8), A );
                break;
            case 0x8E :                                     /* STX 3 */
                MWrite( MRead( PC++ ) + (MRead( PC++ ) << 8 ), X );
                break;
            case 0x8F :
                break;
            case 0x90 :                                     /* BCC */
                if (!P.F.C)
                    PC += MRead( PC );
                PC++;
                break;
            case 0x91 :                                     /* STA 4 */
                addr = MRead( PC++ );
                MWrite( MRead( addr ) + (MRead( addr + 1 ) << 8) + Y, A );
                break;
            case 0x92 :
                break;
            case 0x93 :
                break;
            case 0x94 :                                     /* STY 5 */
                MWrite( MRead( PC++ ) + X, Y );
                break;
            case 0x95 :                                     /* STA 5 */
                MWrite( MRead( PC++ ) + X, A );
                break;
            case 0x96 :                                     /* STX 5 */
                MWrite( MRead( PC++ ) + Y, X );
                break;
            case 0x97 :
                break;
            case 0x98 :                                     /* TYA */
                A = Y;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.Z = !A;
                break;
            case 0x99 :                                     /* STA 6 */
                MWrite( MRead( PC++ ) + (MRead( PC++ ) << 8) + Y, A );
                break;
            case 0x9A :                                     /* TXS */
                SP = X;
                break;
            case 0x9B :
                break;
            case 0x9C :
                break;
            case 0x9D :                                     /* STA 7 */
                MWrite( MRead( PC++ ) + (MRead( PC++ ) << 8) + X, A );
                break;
            case 0x9E :
                break;
            case 0x9F :
                break;
            case 0xA0 :                                     /* LDY 0 */
                Y = MRead( PC++ );
                P.F.Z = !Y;
                P.F.S = (Y & 0x80) ? 1 : 0;
                break;
            case 0xA1 :                                     /* LDA 0 */
                addr = MRead( PC++ ) + X;
                A = MRead( MRead( addr ) + (MRead( addr + 1 ) << 8) );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0xA2 :                                     /* LDX 0 */
                X = MRead( PC++ );
                P.F.Z = !X;
                P.F.S = (X & 0x80) ? 1 : 0;
                break;
            case 0xA3 :
                break;
            case 0xA4 :                                     /* LDY 1 */
                Y = MRead( MRead( PC++ ) );
                P.F.Z = !Y;
                P.F.S = (Y & 0x80) ? 1 : 0;
                break;
            case 0xA5 :                                     /* LDA 1 */
                A = MRead( MRead( PC++ ) );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0xA6 :                                     /* LDX 1 */
                X = MRead( MRead( PC++ ) );
                P.F.Z = !X;
                P.F.S = (X & 0x80) ? 1 : 0;
                break;
            case 0xA7 :
                break;
            case 0xA8 :                                     /* TAY */
                Y = A;
                P.F.S = (Y & 0x80) ? 1 : 0;
                P.F.Z = !Y;
                break;
            case 0xA9 :                                     /* LDA 2 */
                A = MRead( PC++ );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0xAA :                                     /* TAX */
                X = A;
                P.F.S = (X & 0x80) ? 1 : 0;
                P.F.Z = !X;
                break;
            case 0xAB :
                break;
            case 0xAC :                                     /* LDY 3 */
                Y = MRead( MRead( PC++ ) + (MRead( PC++ ) << 8 ) );
                P.F.Z = !Y;
                P.F.S = (Y & 0x80) ? 1 : 0;
                break;
            case 0xAD :                                     /* LDA 3 */
                A = MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0xAE :                                     /* LDX 3 */
                X = MRead( MRead( PC++ ) + (MRead( PC++ ) << 8 ) );
                P.F.Z = !X;
                P.F.S = (X & 0x80) ? 1 : 0;
                break;
            case 0xAF :
                break;
            case 0xB0 :                                     /* BCS */
                if (P.F.C)
                    PC += MRead( PC );
                PC++;
                break;
            case 0xB1 :                                     /* LDA 4 */
                addr = MRead( PC++ );
                A = MRead( MRead( addr ) + (MRead( addr + 1 ) << 8) + Y );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0xB2 :
                break;
            case 0xB3 :
                break;
            case 0xB4 :                                     /* LDY 5 */
                Y = MRead( MRead( PC++ ) + X );
                P.F.Z = !Y;
                P.F.S = (Y & 0x80) ? 1 : 0;
                break;
            case 0xB5 :                                     /* LDA 5 */
                A = MRead( MRead( PC++ ) + X );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0xB6 :                                     /* LDX 5 */
                X = MRead( MRead( PC++ ) + Y );
                P.F.Z = !X;
                P.F.S = (X & 0x80) ? 1 : 0;
                break;
            case 0xB7 :
                break;
            case 0xB8 :                                     /* CLV */
                P.F.V = 0;
                break;
            case 0xB9 :                                     /* LDA 6 */
                A = MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) + Y );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0xBA :                                     /* TSX */
                X = SP;
                P.F.S = (X & 0x80) ? 1 : 0;
                P.F.Z = !X;
                break;
            case 0xBB :
                break;
            case 0xBC :                                     /* LDY 7 */
                Y = MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) + X );
                P.F.Z = !Y;
                P.F.S = (Y & 0x80) ? 1 : 0;
                break;
            case 0xBD :                                     /* LDA 7 */
                A = MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) + X );
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                break;
            case 0xBE :                                     /* LDX 7 */
                X = MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) + Y );
                P.F.Z = !X;
                P.F.S = (X & 0x80) ? 1 : 0;
                break;
            case 0xBF :
                break;
            case 0xC0 :                                     /* CPY 0 */
                tmp = Y - MRead( PC++ );
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.Z = !(tmp & 0xFF);
                break;
            case 0xC1 :                                     /* CMP 0 */
                addr = MRead( PC++ ) + X;
                tmp = A - MRead( MRead( addr ) + (MRead( addr + 1 ) << 8) );
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                break;
            case 0xC2 :
                break;
            case 0xC3 :
                break;
            case 0xC4 :                                     /* CPY 1 */
                tmp = Y - MRead( MRead ( PC++ ) );
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.Z = !(tmp & 0xFF);
                break;
            case 0xC5 :                                     /* CMP 1 */
                tmp = A - MRead( MRead( PC++ ) );
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                break;
            case 0xC6 :                                     /* DEC 1 */
                addr = MRead( PC++ );
                tmp = MRead( addr );
                tmp--;
                MWrite( addr, tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.Z = !(tmp & 0xFF);
                break;
            case 0xC7 :
                break;
            case 0xC8 :                                     /* INY */
                Y++;
                P.F.S = (Y & 0x80) ? 1 : 0;
                P.F.Z = !Y;
                break;
            case 0xC9 :                                     /* CMP 2 */
                tmp = A - MRead( PC++ );
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                break;
            case 0xCA :                                     /* DEX */
                X--;
                P.F.S = (X & 0x80) ? 1 : 0;
                P.F.Z = !X;
                break;
            case 0xCB :
                break;
            case 0xCC :                                     /* CPY 3 */
                tmp = Y - MRead( MRead( PC++ ) + ((MRead( PC++ )) << 8) );
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.Z = !(tmp & 0xFF);
                break;
            case 0xCD :                                     /* CMP 3 */
                tmp = A - MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) );
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                break;
            case 0xCE :                                     /* DEC 3 */
                addr = MRead( PC++ ) + ((MRead( PC++ )) << 8);
                tmp = MRead( addr );
                tmp--;
                MWrite( addr, tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.Z = !(tmp & 0xFF);
                break;
            case 0xCF :
                break;
            case 0xD0 :                                     /* BNE */
                if (!P.F.Z)
                    PC += MRead( PC );
                PC++;
                break;
            case 0xD1 :                                     /* CMP 4 */
                addr = MRead( PC++ );
                tmp = A - MRead( MRead( addr ) + (MRead( addr + 1 ) << 8) + Y );
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                break;
            case 0xD2 :
                break;
            case 0xD3 :
                break;
            case 0xD4 :
                break;
            case 0xD5 :                                     /* CMP 5 */
                tmp = A - MRead( MRead( PC++ ) + X );
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                break;
            case 0xD6 :                                     /* DEC 5 */
                addr = MRead( PC++ ) + X;
                tmp = MRead( addr );
                tmp--;
                MWrite( addr, tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.Z = !(tmp & 0xFF);
                break;
            case 0xD7 :
                break;
            case 0xD8 :                                     /* CLD */
                P.F.D = 0;
                break;
            case 0xD9 :                                     /* CMP 6 */
                tmp = A - MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) + Y );
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                break;
            case 0xDA :
                break;
            case 0xDB :
                break;
            case 0xDC :
                break;
            case 0xDD :                                     /* CMP 7 */
                tmp = A - MRead( MRead( PC++ ) + (MRead( PC++ ) << 8) + X );
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !(tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                break;
            case 0xDE :                                     /* DEC 7 */
                addr = MRead( PC++ ) + ((MRead( PC++ )) << 8) + X;
                tmp = MRead( addr );
                tmp--;
                MWrite( addr, tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.Z = !(tmp & 0xFF);
                break;
            case 0xDF :
                break;
            case 0xE0 :                                     /* CPX 0 */
                tmp = X - MRead( PC++ );
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.Z = !(tmp & 0xFF);
                break;
            case 0xE1 :                                     /* SBC 0 */
                tmp = MRead( PC++ ) + X;
                addr = MRead( MRead( tmp ) + (MRead( tmp + 1 ) << 8) );
                if (P.F.D)
                {
                    w1 = (A & 0x0F) - ((addr & 0x0F) + !P.F.C);
                    if (w1 > 9 || w1 < 0)
                        w1 -= 6;
                    w2 = (A >> 4) - ((addr >> 4) + (w1 < 0));
                    if (w2 > 9 || w2 < 0)
                        w2 -= 6;
                    tmp = (w2 << 4) + (w1 & 0x0F);
                }
                else
                    tmp = A - (addr + !P.F.C);
                P.F.V = ((A & 0xC0) == 0x80) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = (A & 0x80) ^ P.F.V;
                break;
            case 0xE2 :
                break;
            case 0xE3 :
                break;
            case 0xE4 :                                     /* CPX 1 */
                tmp = X - MRead( MRead ( PC++ ) );
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.Z = !(tmp & 0xFF);
                break;
            case 0xE5 :                                     /* SBC 1 */
                addr = MRead( MRead( PC++ ) );
                if (P.F.D)
                {
                    w1 = (A & 0x0F) - ((addr & 0x0F) + !P.F.C);
                    if (w1 > 9 || w1 < 0)
                        w1 -= 6;
                    w2 = (A >> 4) - ((addr >> 4) + (w1 < 0));
                    if (w2 > 9 || w2 < 0)
                        w2 -= 6;
                    tmp = (w2 << 4) + (w1 & 0x0F);
                }
                else
                    tmp = A - (addr + !P.F.C);
                P.F.V = ((A & 0xC0) == 0x80) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = (A & 0x80) ^ P.F.V;
                break;
            case 0xE6 :                                     /* INC 1 */
                addr = MRead( PC++ );
                tmp = MRead( addr );
                tmp++;
                MWrite( addr, tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.Z = !(tmp & 0xFF);
                break;
            case 0xE7 :
                break;
            case 0xE8 :                                     /* INX */
                X++;
                P.F.S = (X & 0x80) ? 1 : 0;
                P.F.Z = !X;
                break;
            case 0xE9 :                                     /* SBC 2 */
                addr = MRead( PC++ );
                if (P.F.D)
                {
                    w1 = (A & 0x0F) - ((addr & 0x0F) + !P.F.C);
                    if (w1 > 9 || w1 < 0)
                        w1 -= 6;
                    w2 = (A >> 4) - ((addr >> 4) + (w1 < 0));
                    if (w2 > 9 || w2 < 0)
                        w2 -= 6;
                    tmp = (w2 << 4) + (w1 & 0x0F);
                }
                else
                    tmp = A - (addr + !P.F.C);
                P.F.V = ((A & 0xC0) == 0x80) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = (A & 0x80) ^ P.F.V;
                break;
            case 0xEA :                                     /* NOP */
                break;
            case 0xEB :
                break;
            case 0xEC :                                     /* CPX 3 */
                tmp = X - MRead( MRead( PC++ ) + ((MRead( PC++ )) << 8) );
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.Z = !(tmp & 0xFF);
                break;
            case 0xED :                                     /* SBC 3 */
                addr = MRead( MRead( PC++ ) + ((MRead( PC++ )) << 8) );
                if (P.F.D)
                {
                    w1 = (A & 0x0F) - ((addr & 0x0F) + !P.F.C);
                    if (w1 > 9 || w1 < 0)
                        w1 -= 6;
                    w2 = (A >> 4) - ((addr >> 4) + (w1 < 0));
                    if (w2 > 9 || w2 < 0)
                        w2 -= 6;
                    tmp = (w2 << 4) + (w1 & 0x0F);
                }
                else
                    tmp = A - (addr + !P.F.C);
                P.F.V = ((A & 0xC0) == 0x80) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = (A & 0x80) ^ P.F.V;
                break;
            case 0xEE :                                     /* INC 3 */
                addr = MRead( PC++ ) + ((MRead( PC++ )) << 8);
                tmp = MRead( addr );
                tmp++;
                MWrite( addr, tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.Z = !(tmp & 0xFF);
                break;
            case 0xEF :
                break;
            case 0xF0 :                                     /* BEQ */
                if (P.F.Z)
                    PC += MRead( PC );
                PC++;
                break;
            case 0xF1 :                                     /* SBC 4 */
                tmp = MRead( PC++ );
                addr = MRead( MRead( tmp ) + ((MRead( tmp + 1 )) << 8) + Y );
                if (P.F.D)
                {
                    w1 = (A & 0x0F) - ((addr & 0x0F) + !P.F.C);
                    if (w1 > 9 || w1 < 0)
                        w1 -= 6;
                    w2 = (A >> 4) - ((addr >> 4) + (w1 < 0));
                    if (w2 > 9 || w2 < 0)
                        w2 -= 6;
                    tmp = (w2 << 4) + (w1 & 0x0F);
                }
                else
                    tmp = A - (addr + !P.F.C);
                P.F.V = ((A & 0xC0) == 0x80) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = (A & 0x80) ^ P.F.V;
                break;
            case 0xF2 :
                break;
            case 0xF3 :
                break;
            case 0xF4 :
                break;
            case 0xF5 :                                     /* SBC 5 */
                addr = MRead( MRead( PC++ ) + X );
                if (P.F.D)
                {
                    w1 = (A & 0x0F) - ((addr & 0x0F) + !P.F.C);
                    if (w1 > 9 || w1 < 0)
                        w1 -= 6;
                    w2 = (A >> 4) - ((addr >> 4) + (w1 < 0));
                    if (w2 > 9 || w2 < 0)
                        w2 -= 6;
                    tmp = (w2 << 4) + (w1 & 0x0F);
                }
                else
                    tmp = A - (addr + !P.F.C);
                P.F.V = ((A & 0xC0) == 0x80) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = (A & 0x80) ^ P.F.V;
                break;
            case 0xF6 :                                     /* INC 5 */
                addr = MRead( PC++ ) + X;
                tmp = MRead( addr );
                tmp++;
                MWrite( addr, tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.Z = !(tmp & 0xFF);
                break;
            case 0xF7 :
                break;
            case 0xF8 :                                     /* SED */
                P.F.D = 1;
                break;
            case 0xF9 :                                     /* SBC 6 */
                addr = MRead( MRead( PC++ ) + ((MRead( PC++ )) << 8 ) + Y );
                if (P.F.D)
                {
                    w1 = (A & 0x0F) - ((addr & 0x0F) + !P.F.C);
                    if (w1 > 9 || w1 < 0)
                        w1 -= 6;
                    w2 = (A >> 4) - ((addr >> 4) + (w1 < 0));
                    if (w2 > 9 || w2 < 0)
                        w2 -= 6;
                    tmp = (w2 << 4) + (w1 & 0x0F);
                }
                else
                    tmp = A - (addr + !P.F.C);
                P.F.V = ((A & 0xC0) == 0x80) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = (A & 0x80) ^ P.F.V;
                break;
            case 0xFA :
                break;
            case 0xFB :
                break;
            case 0xFC :
                break;
            case 0xFD :                                     /* SBC 7 */
                addr = MRead( MRead( PC++ ) + ((MRead( PC++ )) << 8) + X );
                if (P.F.D)
                {
                    w1 = (A & 0x0F) - ((addr & 0x0F) + !P.F.C);
                    if (w1 > 9 || w1 < 0)
                        w1 -= 6;
                    w2 = (A >> 4) - ((addr >> 4) + (w1 < 0));
                    if (w2 > 9 || w2 < 0)
                        w2 -= 6;
                    tmp = (w2 << 4) + (w1 & 0x0F);
                }
                else
                    tmp = A - (addr + !P.F.C);
                P.F.V = ((A & 0xC0) == 0x80) ? 1 : 0;
                A = tmp & 0xFF;
                P.F.C = (tmp > 0xFF) ? 0 : 1;
                P.F.Z = !A;
                P.F.S = (A & 0x80) ? 1 : 0;
                P.F.V = (A & 0x80) ^ P.F.V;
                break;
            case 0xFE :                                     /* INC 7 */
                addr = MRead( PC++ ) + ((MRead( PC++ )) << 8) + X;
                tmp = MRead( addr );
                tmp++;
                MWrite( addr, tmp & 0xFF);
                P.F.S = (tmp & 0x80) ? 1 : 0;
                P.F.Z = !(tmp & 0xFF);
                break;
            case 0xFF :
                break;
        }
    }
}
