#include <stdio.h>
#include <stdlib.h>
#include <bios.h>
#include <dir.h>

main( int argc, char **argv )
{
    char fname[80];

    if ( argc == 1 )
        strcpy( fname, "*.*" );
    else
        strcpy( fname, argv[1] );
    if (GetFile( fname ))
        printf("No files found\n");
    else
    {
        clrscr();
        printf( "%s\n", fname );
    }
}

/********************** Select filename **************************/

struct dl               /* Structure for storing directory entries */
{
    struct dl *fp;      /* forward pointer */
    struct dl *bp;      /* backward pointer */
    int l;              /* display line */
    int p;              /* display position */
    int n;              /* last entry on page */
    char fn[13];        /* filename to display */
} *begin, *end, *first, *last, *rec, *prev;

GetFile( char *fname )
{
    struct ffblk ffblk;
    int done, count = 0, flags, line = 3, pos = 1, point, stop = 0;
    unsigned k;
    char drive[MAXDRIVE],
         dir[MAXDIR],
         file[MAXFILE],
         ext[MAXEXT],
         work[MAXPATH];

    flags = fnsplit(fname, drive, dir, file, ext);
    if ( !((flags & FILENAME) && (flags & EXTENSION)))
    {
        strcpy(file, "*");
        strcpy(ext, ".*");
    }
    fnmerge(fname, drive, dir, file, ext);
    clrscr();
    printf("PATH = %s\n\n", fname);

    if (done = findfirst( fname, &ffblk, 0 ))
        return(1);
    while ( !done )
    {
        prev = rec;
        if ((end = rec = malloc( sizeof( struct dl ))) == NULL)
        {
            fprintf(stderr, "\nOut of memory\n");
            prev = begin;
            do
            {
                rec = prev;
                prev = rec->fp;
                free(rec);
            }
            while (prev->fp != NULL);
            return(1);
        }

        if ( !count )
        {
            begin = rec;
            prev = NULL;
        }
        else
        {
            rec->bp = prev;
            prev->fp = rec;
        }
        rec->bp = prev;
        strcpy( rec->fn, ffblk.ff_name );
        rec->fp = NULL;
        rec->l = line++;
        rec->p = pos;
        rec->n = 0;
        if (line > 24)
        {
            line = 3;
            if (( pos += 15 ) > 68 )
            {
                pos = 1;
                rec->n = 1;
            }
        }
        count++;
        done = findnext( &ffblk );
    }

    prev = end = rec = begin;

    GetFilePage();

    end = rec;
    rec = prev;

    while (!stop)
    {
        gotoxy( prev->p, prev->l );
        printf(" %s", prev->fn);
        gotoxy( rec->p, rec->l );
        printf(">%s", rec->fn);
        prev = rec;
        k = bioskey(0) >> 8;
        switch(k)
        {
            case 0x01 :                         /* Esc */
                stop = 1;
                fname[0] = '\0';
                break;
            case 0x1C :                         /* Return */
                stop = 1;
                fnsplit(rec->fn, work, work, file, ext);
                fnmerge(fname, drive, dir, file, ext);
                break;
            case 0x48 :                         /* CurUp */
                if (rec != first)
                    rec = rec->bp;
                break;
            case 0x50 :                         /* CurDown */
                if (rec != last)
                    rec = rec->fp;
                break;
            case 0x4B :                         /* CurLeft */
                line = rec->l;
                if (rec->p > 1)
                    do
                    {
                        if (rec == first)
                            break;
                        rec = rec->bp;
                    }
                    while (rec->l != line);
                break;
            case 0x4D :                         /* CurRight */
                line = rec->l;
                if (rec->p < last->p)
                    do
                    {
                        if (rec == last)
                            break;
                        rec = rec->fp;
                    }
                    while (rec->l != line);
                break;
            case 0x47 :                         /* Home */
                rec = first;
                break;
            case 0x4F :                         /* End */
                rec = last;
                break;
            case 0x49 :                         /* PgUp */
                if ( first->bp != NULL )
                {
                    rec = first->bp;
                    while (rec->bp != NULL)
                    {
                        rec = rec->bp;
                        if (rec->n)
                            break;
                    }
                    GetFilePage( );
                    prev = rec = first;
                }
                break;
            case 0x51 :                         /* PgDn */
                if ( last->fp != NULL )
                {
                    rec = last->fp;
                    GetFilePage( );
                    prev = rec = first;
                }
                break;
        }
    }

    /*  Release allocated memory */
    prev = begin;
    do
    {
        rec = prev;
        prev = rec->fp;
        free(rec);
    }
    while (prev->fp != NULL);

    return(0);
}

GetFilePage( )
{
    clrscr();

    first = rec;
    while ( rec != NULL)
    {
        gotoxy(rec->p, rec->l);
        printf( " %s", rec->fn );
        last = rec;
        if (rec->n)
            break;
        rec = rec->fp;
    }
    return(0);
}
